@@ -2,12 +2,10 @@
 
 #
 # USAGE: ./dist.sh -v VERSION -r REVISION [-rs REVISION-SVN] [-pr REPOS-PATH]
-#                  [-apr APR-PATH] [-apu APR-UTIL-PATH] 
-#                  [-api APR-ICONV-PATH] [-neon NEON-PATH]
-#                  [-apr-tag APR-TAG] [-apu-tag APR-UTIL-TAG] 
-#                  [-api-tag APR-ICONV-TAG] [-neon-tag NEON-TAG]
-#                  [-zip] [-sign] [-fetch]
+#                  [-apr PATH-TO-APR ] [-apru PATH-TO-APR-UTIL] 
+#                  [-apri PATH-TO-APR-ICONV] [-neon PATH-TO-NEON]
 #                  [-alpha ALPHA_NUM|-beta BETA_NUM|-rc RC_NUM]
+#                  [-zip] [-sign] [-nodeps]
 #
 #   Create a distribution tarball, labelling it with the given VERSION.
 #   The REVISION or REVISION-SVN will be used in the version string.
@@ -23,9 +21,9 @@
 #   specify the path to them with the -apr, -apru or -neon options.
 #   For example:
 #      ./dist.sh -v 1.1.0 -r 10277 -pr branches/1.1.x \
-#        -neon ~/in-tree-libraries/neon-0.24.7 \
+#        -apr -neon ~/in-tree-libraries/neon-0.24.7 \
 #        -apr ~/in-tree-libraries/httpd-2.0.50/srclib/apr \
-#        -apu ~/in-tree-libraries/httpd-2.0.50/srclib/apr-util/
+#        -apru ~/in-tree-libraries/httpd-2.0.50/srclib/apr-util/
 #
 #   When building a alpha, beta or rc tarballs pass the apppropriate flag
 #   followeb by the number for that releasse.  For example you'd do
@@ -40,13 +38,11 @@
 
 
 # A quick and dirty usage message
-USAGE="USAGE: ./dist.sh -v VERSION -r REVISION
-[-rs REVISION-SVN] [-pr REPOS-PATH]
-[-alpha ALPHA_NUM|-beta BETA_NUM|-rc RC_NUM]
-[-apr APR_DIR] [-apu APR_UTIL_DIR] [-api APR_ICONV_DIR] [-neon NEON_DIR]
-[-apr-tag APR_TAG] [-apu-tag APR_UTIL_TAG] [-api-tag APR_ICONV_TAG]
-[-neon-tag NEON_TAG]
-[-zip] [-sign] [-fetch]
+USAGE="USAGE: ./dist.sh -v VERSION -r REVISION \
+[-rs REVISION-SVN ] [-pr REPOS-PATH] \
+[-alpha ALPHA_NUM|-beta BETA_NUM|-rc RC_NUM] \
+[-apr APR_PATH ] [-apru APR_UTIL_PATH] [-apri APR_ICONV_PATH] \
+[-neon NEON_PATH ] [-zip] [-sign] [-nodeps]
  EXAMPLES: ./dist.sh -v 0.36.0 -r 8278
            ./dist.sh -v 0.36.0 -r 8278 -pr trunk
            ./dist.sh -v 0.36.0 -r 8282 -rs 8278 -pr tags/0.36.0
@@ -65,15 +61,11 @@ do
          -r)  REVISION="$ARG" ;;
         -rs)  REVISION_SVN="$ARG" ;;
         -pr)  REPOS_PATH="$ARG" ;;
-        -rc)  RC="$ARG" ;;
+	-rc)  RC="$ARG" ;;
        -apr)  APR_PATH="$ARG" ;;
-       -apu)  APU_PATH="$ARG" ;;
-       -api)  API_PATH="$ARG" ;;
-   -apr-tag)  APR_TAG="$ARG" ;;
-   -apu-tag)  APU_TAG="$ARG" ;;
-   -api-tag)  API_TAG="$ARG" ;;
+       -apru) APRU_PATH="$ARG" ;;
+       -apri) APRI_PATH="$ARG" ;;
       -neon)  NEON_PATH="$ARG" ;;
-  -neon-tag)  NEON_TAG="$ARG" ;;
       -beta)  BETA="$ARG" ;;
      -alpha)  ALPHA="$ARG" ;;
           *)  ARG_PREV=$ARG ;;
@@ -84,21 +76,21 @@ do
   else
 
     case $ARG in
-      -v|-r|-rs|-pr|-beta|-rc|-alpha|-apr|-apr-tag|-apu|-apu-tag|-api|-api-tag|-neon|-neon-tag)
+      -v|-r|-rs|-pr|-beta|-rc|-alpha|-apr|-apru|-apri|-neon)
         ARG_PREV=$ARG
         ;;
-      -fetch)
-        FETCH=1
+      -zip)
+        ZIP=1
+        ARG_PREV=""
+        ;;
+      -nodeps)
+        NODEPS=1
         ARG_PREV=""
         ;;
       -sign)
         SIGN=1
         ARG_PREV=""
         ;;
-      -zip)
-        ZIP=1
-        ARG_PREV=""
-        ;;
       *)
         echo " $USAGE"
         exit 1
@@ -139,54 +131,20 @@ if [ -z "$VERSION" ] || [ -z "$REVISION" ] ; then
   exit 1
 fi
 
-if [ -n "$FETCH" ]; then
-  if [ -z "$APR_PATH" -a -z "$APR_TAG" ]; then
-    APR_TAG='0.9.6'
-  fi
-  if [ -z "$APU_PATH" -a -z "$APU_TAG" ]; then
-    APU_TAG='0.9.6'
-  fi
-  if [ -z "$API_PATH" -a -z "$API_PATH" ]; then
-    API_TAG='0.9.6'
-  fi
-  if [ -z "$NEON_PATH" -a -z "$NEON_TAG" ]; then
-    NEON_TAG='0.24.7'
-  fi
-else
-  if [ -z "$APR_PATH" -a -z "$APR_TAG" ]; then
-    APR_PATH='apr'
-  fi
-  if [ -z "$APU_PATH" -a -z "$APU_TAG" ]; then
-    APU_PATH='apr-util'
-  fi
-  if [ -z "$API_PATH" -a -z "$API_PATH" ]; then
-    API_PATH='apr-iconv'
-  fi
-  if [ -z "$NEON_PATH" -a -z "$NEON_TAG" ]; then
-    NEON_PATH='neon'
-  fi
-
+if [ -z "$APR_PATH" ]; then
+  APR_PATH='apr'
 fi
 
-if [ -z "$APR_TAG" -a ! -d "$APR_PATH" ]; then
-  echo "Missing $APR_PATH directory.  Aborting."
-  echo " $USAGE"
-  exit 2
+if [ -z "$APRU_PATH" ]; then
+  APRU_PATH='apr-util'
 fi
-if [ -z "$APU_TAG" -a ! -d "$APU_PATH" ]; then
-  echo "Missing $APU_PATH directory.  Aborting."
-  echo " $USAGE"
-  exit 2
+
+if [ -z "$NEON_PATH" ]; then
+  NEON_PATH='neon'
 fi
-if [ -n "$ZIP" -a -z "$API_TAG" -a ! -d "$API_PATH" ]; then
-  echo "Missing $API_PATH directory.  Aborting."
-  echo " $USAGE"
-  exit 2
-fi
-if [ -z "$NEON_TAG" -a ! -d "$NEON_PATH" ]; then
-  echo "Missing $NEON_PATH directory.  Aborting."
-  echo " $USAGE"
-  exit 2
+
+if [ -z "$APRI_PATH" ]; then
+  APRI_PATH='apr-iconv'
 fi
 
 if [ -z "$REPOS_PATH" ]; then
@@ -195,12 +153,14 @@ else
   REPOS_PATH="`echo $REPOS_PATH | sed 's/^\/*//'`"
 fi
 
+# See comment when we 'roll' the tarballs as to why pax is required.
 type pax > /dev/null 2>&1
 if [ $? -ne 0 ] && [ -z "$ZIP" ]; then
   echo "ERROR: pax could not be found"
   exit 1
 fi
 
+# Default to 'wget', but allow 'curl' to be used if available.
 HTTP_FETCH=wget
 HTTP_FETCH_OUTPUT="-O"
 type wget > /dev/null 2>&1
@@ -236,12 +196,12 @@ echo "Exporting revision $REVISION of Subversion into sandbox..."
 install_dependency()
 {
   DEP_NAME=$1
-  DEP_TAG=$2
-  if [ -z $3 ]; then
+  if [ -z $2 ]; then
     DEP_PATH=/dev/null
   else
-    DEP_PATH=$3
+    DEP_PATH=$2
   fi
+
   if [ -d $DEP_PATH ]; then
     if [ -d $DEP_PATH/.svn ]; then
       echo "Exporting local $DEP_NAME into sandbox"
@@ -249,61 +209,37 @@ install_dependency()
     else
       echo "Copying local $DEP_NAME into sandbox"
       cp -r "$DEP_PATH" "$DISTPATH/$DEP_NAME" 
-      (cd "$DISTPATH/$DEP_NAME" && [ -f Makefile ] && make extraclean)
+      (cd "$DISTPATH/$DEP_NAME" && [ -f Makefile ] && make distclean)
       echo "Removing all CVS/ and .cvsignore files from $DEP_NAME..."
       find "$DISTPATH/$DEP_NAME" -name CVS -type d -print | xargs rm -fr
       find "$DISTPATH/$DEP_NAME" -name .cvsignore -print | xargs rm -f
+      find "$DISTPATH/$DEP_NAME" -name '*.o' -print | xargs rm -f
     fi
   else
-    echo "Exporting $DEP_NAME into sandbox ($DEP_TAG)"
-    ${SVN:-svn} export -q $EXTRA_EXPORT_OPTIONS $DEP_TAG "$DISTPATH/$DEP_NAME"
+    # Not having the dependency directories isn't fatal if -nodeps passed.
+    if [ -z "$NODEPS" ]; then
+      echo "Missing dependency directory!"
+      exit 2
+    fi
   fi
 }
 
-install_dependency apr http://svn.apache.org/repos/asf/apr/apr/tags/${APR_TAG} "$APR_PATH"
-install_dependency apr-util http://svn.apache.org/repos/asf/apr/apr-util/tags/${APU_TAG} "$APU_PATH"
+install_dependency apr "$APR_PATH"
+install_dependency apr-util "$APRU_PATH"
 
 if [ -n "$ZIP" ]; then
-  install_dependency apr-iconv http://svn.apache.org/repos/asf/apr/apr-iconv/tags/${API_TAG} "$API_PATH"
+  install_dependency apr-iconv "$APRI_PATH"
 fi
 
-install_dependency neon http://svn.webdav.org/repos/projects/neon/tags/${NEON_TAG} "$NEON_PATH"
+install_dependency neon "$NEON_PATH"
 
 find "$DISTPATH" -name config.nice -print | xargs rm -f
 
-echo "Running ./autogen.sh in sandbox, to create ./configure ..."
-(cd "$DISTPATH" && ./autogen.sh --release) || exit 1
-
-if [ ! -f $DISTPATH/neon/configure ]; then
-  echo "Creating neon configure"
-  (cd "$DISTPATH/neon" && ./autogen.sh) || exit 1
-fi
-
-echo "Removing any autom4te.cache directories that might exist..."
-find "$DISTPATH" -depth -type d -name 'autom4te*.cache' -exec rm -rf {} \;
-
-echo "Downloading book into sandbox..."
-
-BOOK_PDF=http://svnbook.red-bean.com/en/1.1/svn-book.pdf
-BOOK_PDF_DEST="$DISTPATH/doc/book/book/svn-book.pdf"
-BOOK_HTML=http://svnbook.red-bean.com/en/1.1/svn-book.html
-BOOK_HTML_DEST="$DISTPATH/doc/book/book/svn-book.html"
-
-$HTTP_FETCH $BOOK_PDF $HTTP_FETCH_OUTPUT $BOOK_PDF_DEST ||
-  ( echo "ERROR: Problem getting the svn-book.pdf file." && exit 1 )
-
-$HTTP_FETCH $BOOK_HTML $HTTP_FETCH_OUTPUT $BOOK_HTML_DEST ||
-  ( echo "ERROR: Problem getting the svn-book.html file." && exit 1 )
-
-cat > "$DISTPATH/ChangeLog.CVS" <<EOF
-The old CVS ChangeLog is kept at 
-
-     http://subversion.tigris.org/
-
-If you want to see changes since Subversion went self-hosting,
-you probably want to use the "svn log" command -- and if it 
-does not do what you need, please send in a patch!
-EOF
+# Massage the new version number into svn_version.h.  We need to do
+# this before running autogen.sh --release on the subversion code,
+# because otherwise svn_version.h's mtime makes SWIG files regenerate
+# on end-user's systems, when they should just be compiled by the
+# Release Manager and left at that.
 
 ver_major=`echo $VERSION | cut -d '.' -f 1`
 ver_minor=`echo $VERSION | cut -d '.' -f 2`
@@ -325,6 +261,27 @@ mv -f "$vsn_file.tmp" "$vsn_file"
 
 cp "$vsn_file" "svn_version.h.dist"
 
+echo "Running ./autogen.sh in sandbox, to create ./configure ..."
+(cd "$DISTPATH" && ./autogen.sh --release) || exit 1
+
+if [ ! -f $DISTPATH/neon/configure ]; then
+  echo "Creating neon configure"
+  (cd "$DISTPATH/neon" && ./autogen.sh) || exit 1
+fi
+
+echo "Removing any autom4te.cache directories that might exist..."
+find "$DISTPATH" -depth -type d -name 'autom4te*.cache' -exec rm -rf {} \;
+
+cat > "$DISTPATH/ChangeLog.CVS" <<EOF
+The old CVS ChangeLog is kept at 
+
+     http://subversion.tigris.org/
+
+If you want to see changes since Subversion went self-hosting,
+you probably want to use the "svn log" command -- and if it 
+does not do what you need, please send in a patch!
+EOF
+
 if [ -z "$ZIP" ]; then
   # Do not use tar, it's probably GNU tar which produces tar files that are
   # not compliant with POSIX.1 when including filenames longer than 100 chars.
@@ -338,15 +295,24 @@ if [ -z "$ZIP" ]; then
   echo "Compressing to $DISTNAME.tar.bz2 ..."
   bzip2 -9fk "$DISTNAME.tar"
 
+  # Use the gzip -n flag - this prevents it from storing the original name of
+  # the .tar file, and far more importantly, the mtime of the .tar file, in the
+  # produced .tar.gz file. This is important, because it makes the gzip
+  # encoding reproducable by anyone else who has an similar version of gzip,
+  # and also uses "gzip -9n". This means that committers who want to GPG-sign
+  # both the .tar.gz and the .tar.bz2 can download the .tar.bz2 (which is
+  # smaller), and locally generate an exact duplicate of the official .tar.gz
+  # file. This metadata is data on the temporary uncompressed tarball itself,
+  # not any of its contents, so there will be no effect on end-users.
   echo "Compressing to $DISTNAME.tar.gz ..."
-  gzip -9f "$DISTNAME.tar"
+  gzip -9nf "$DISTNAME.tar"
 else
   echo "Rolling $DISTNAME.zip ..."
   (cd "$DIST_SANDBOX" > /dev/null && zip -q -r - "$DISTNAME") > \
     "$DISTNAME.zip"
 fi
-#echo "Removing sandbox..."
-#rm -rf "$DIST_SANDBOX"
+echo "Removing sandbox..."
+rm -rf "$DIST_SANDBOX"
 
 sign_file()
 {
@@ -378,16 +344,16 @@ sign_file()
 echo ""
 echo "Done:"
 if [ -z "$ZIP" ]; then
-  ls -l "$DISTNAME.tar.gz" "$DISTNAME.tar.bz2"
+  ls -l "$DISTNAME.tar.bz2" "$DISTNAME.tar.gz"
   sign_file $DISTNAME.tar.gz $DISTNAME.tar.bz2
   echo ""
   echo "md5sums:"
-  md5sum "$DISTNAME.tar.gz" "$DISTNAME.tar.bz2"
+  md5sum "$DISTNAME.tar.bz2" "$DISTNAME.tar.gz"
   type sha1sum > /dev/null 2>&1
   if [ $? -eq 0 ]; then
     echo ""
     echo "sha1sums:"
-    sha1sum "$DISTNAME.tar.gz" "$DISTNAME.tar.bz2"
+    sha1sum "$DISTNAME.tar.bz2" "$DISTNAME.tar.gz"
   fi
 else
   ls -l "$DISTNAME.zip"
