@@ -3,7 +3,7 @@ package org.tigris.subversion.javahl;
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2003-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2003-2005 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -17,6 +17,9 @@ package org.tigris.subversion.javahl;
  * ====================================================================
  * @endcopyright
  */
+
+import java.io.OutputStream;
+
 /**
  * This is the main interface class. All subversion commandline client svn &
  * svnversion operation are implemented in this class. This class is not
@@ -29,56 +32,9 @@ public class SVNClient implements SVNClientInterface
      */
     static
     {
-        loadNativeLibrary();
+        NativeResources.loadNativeLibrary();
     }
 
-    /**
-     * Called upon class initialization.
-     *
-     * @throws UnsatisfiedLinkError If the native library cannot be
-     * loaded.
-     */
-    private static void loadNativeLibrary()
-    {
-        // If the user specified the fully qualified path to the
-        // native library, try loading that first.
-        try
-        {
-            String specifiedLibraryName =
-                System.getProperty("subversion.native.library");
-            if (specifiedLibraryName != null)
-            {
-                System.load(specifiedLibraryName);
-                return;
-            }
-        }
-        catch (UnsatisfiedLinkError ex)
-        {
-            // ignore that error to try again
-        }
-
-        // Try to load the library by its name.  Failing that, try to
-        // load it by its old name.
-        try
-        {
-            System.loadLibrary("svnjavahl-1");
-            return;
-        }
-        catch (UnsatisfiedLinkError ex)
-        {
-            try
-            {
-                System.loadLibrary("libsvnjavahl-1");
-                return;
-            }
-            catch (UnsatisfiedLinkError e)
-            {
-                System.loadLibrary("svnjavahl");
-            }
-        }
-    }
-
-
     /**
      * Standard empty contructor, builds just the native peer.
      */
@@ -104,6 +60,23 @@ public class SVNClient implements SVNClientInterface
      * of this member
      */
     protected long cppAddr;
+    /**
+     * @return The name of the working copy's administrative
+     * directory, which is usually <code>.svn</code>.
+     * @see <a
+     * href="http://svn.collab.net/repos/svn/trunk/notes/asp-dot-net-hack.txt">Instructions</a>
+     * on changing this as a work-around for the behavior of ASP.Net
+     * on Windows.
+     * @since 1.3
+     */
+    public native String getAdminDirectoryName();
+    /**
+     * @param name The name of the directory to compare.
+     * @return Whether <code>name</code> is that of a working copy
+     * administrative directory.
+     * @since 1.3
+     */
+    public native boolean isAdminDirectory(String name);
     /**
       * Returns the last destination path submitted.
       * @deprecated
@@ -119,8 +92,8 @@ public class SVNClient implements SVNClientInterface
      * @param getAll    get status for uninteristing files (unchanged).
      * @return Array of Status entries.
      */
-    public Status[]status(String path, boolean descend, boolean onServer,
-                          boolean getAll) throws ClientException
+    public Status[] status(String path, boolean descend, boolean onServer,
+                           boolean getAll) throws ClientException
     {
         return status(path, descend, onServer, getAll, false);
     }
@@ -166,7 +139,7 @@ public class SVNClient implements SVNClientInterface
      * @param recurse   recurse into subdirectories
      * @return  Array of DirEntry objects.
      */
-    public DirEntry[]list(String url, Revision revision, boolean recurse)
+    public DirEntry[] list(String url, Revision revision, boolean recurse)
             throws ClientException
     {
         return list(url, revision, revision, recurse);
@@ -196,8 +169,8 @@ public class SVNClient implements SVNClientInterface
     public native Status singleStatus(String path, boolean onServer)
             throws ClientException;
     /**
-     * Sets the username used for authentification.
-     * @param username  the username
+     * Sets the user name used for authentification.
+     * @param username The user name.
      */
     public native void username(String username);
     /**
@@ -206,7 +179,12 @@ public class SVNClient implements SVNClientInterface
      */
     public native void password(String password);
     /**
-     * Register callback interface to supply username and password on demand
+     * Register callback interface to supply user name and password on
+     * demand.  This callback can also be used to provide the
+     * equivalent of the <code>--no-auth-cache</code> and
+     * <code>--non-interactive</code> arguments accepted by the
+     * command-line client.
+     *
      * @param prompt the callback interface
      */
     public native void setPrompt(PromptUserPassword prompt);
@@ -935,6 +913,24 @@ public class SVNClient implements SVNClientInterface
                                      Revision pegRevision)
             throws ClientException;
 
+    /**
+     * Write the file's content to the specified output stream.  If
+     * you need an InputStream, use a
+     * PipedInputStream/PipedOutputStream combination.
+     *
+     * @param path        the path of the file
+     * @param revision    the revision to retrieve
+     * @param pegRevision the revision at which to interpret the path
+     * @param stream      the stream to write the file's content to
+     * @throws ClientException
+     * @see <a href="http://java.sun.com/j2se/1.4.2/docs/api/java/io/PipedOutputStream.html">PipedOutputStream</a>
+     * @see <a href="http://java.sun.com/j2se/1.4.2/docs/api/java/io/PipedInputStream.html">PipedInputStream</a>
+     */
+    public native void streamFileContent(String path, Revision revision, 
+                            Revision pegRevision, int bufferSize, 
+                            OutputStream stream) 
+            throws ClientException;    
+
     /**
      * Rewrite the url's in the working copy
      * @param from      old url
@@ -1120,4 +1116,9 @@ public class SVNClient implements SVNClientInterface
     public native Info2[] info2(String pathOrUrl, Revision revision,
                                 Revision pegRevision, boolean recurse)
             throws ClientException;
+    /**
+     * Internal method to initialize the native layer. Only to be called by
+     * NativeResources.loadNativeLibrary
+     */
+    static native void initNative();
 }
