@@ -23,22 +23,15 @@
 #include <assert.h>
 #include <apr_general.h>  /* for strcasecmp() */
 #include <apr_pools.h>
-#include <apr_hash.h>
-#include <apr_tables.h>
 #include <apr_file_io.h>
 #include <apr_strings.h>
-#include <apr_lib.h>
 #include "svn_types.h"
-#include "svn_delta.h"
 #include "svn_string.h"
-#include "svn_time.h"
 #include "svn_path.h"
-#include "svn_xml.h"
 #include "svn_error.h"
-#include "svn_utf.h"
 #include "svn_subst.h"
 #include "svn_io.h"
-#include "svn_hash.h"
+#include "svn_props.h"
 #include "svn_wc.h"
 
 #include "wc.h"
@@ -56,7 +49,7 @@ svn_wc_translated_file (const char **xlated_p,
 {
   svn_subst_eol_style_t style;
   const char *eol;
-  svn_subst_keywords_t *keywords;
+  apr_hash_t *keywords;
   svn_boolean_t special;
   
   SVN_ERR (svn_wc__get_eol_style (&style, &eol, vfile, adm_access, pool));
@@ -93,7 +86,7 @@ svn_wc_translated_file (const char **xlated_p,
       
       if (style == svn_subst_eol_style_fixed)
         {
-          SVN_ERR (svn_subst_copy_and_translate2 (vfile,
+          SVN_ERR (svn_subst_copy_and_translate3 (vfile,
                                                   tmp_vfile,
                                                   eol,
                                                   TRUE,
@@ -104,7 +97,7 @@ svn_wc_translated_file (const char **xlated_p,
         }
       else if (style == svn_subst_eol_style_native)
         {
-          SVN_ERR (svn_subst_copy_and_translate2 (vfile,
+          SVN_ERR (svn_subst_copy_and_translate3 (vfile,
                                                   tmp_vfile,
                                                   SVN_WC__DEFAULT_EOL_MARKER,
                                                   force_repair,
@@ -115,7 +108,7 @@ svn_wc_translated_file (const char **xlated_p,
         }
       else if (style == svn_subst_eol_style_none)
         {
-          SVN_ERR (svn_subst_copy_and_translate2 (vfile,
+          SVN_ERR (svn_subst_copy_and_translate3 (vfile,
                                                   tmp_vfile,
                                                   NULL,
                                                   force_repair,
@@ -129,7 +122,7 @@ svn_wc_translated_file (const char **xlated_p,
           return svn_error_createf
             (SVN_ERR_IO_UNKNOWN_EOL, NULL,
              _("'%s' has unknown value for svn:eol-style property"),
-             vfile);
+             svn_path_local_style (vfile, pool));
         }
 
       *xlated_p = tmp_vfile;
@@ -176,19 +169,15 @@ svn_wc__eol_value_from_string (const char **value, const char *eol)
 
 
 svn_error_t *
-svn_wc__get_keywords (svn_subst_keywords_t **keywords,
+svn_wc__get_keywords (apr_hash_t **keywords,
                       const char *path,
                       svn_wc_adm_access_t *adm_access,
                       const char *force_list,
                       apr_pool_t *pool)
 {
   const char *list;
-  svn_subst_keywords_t tmp_keywords = { 0 };
   const svn_wc_entry_t *entry = NULL;
 
-  /* Start by assuming no keywords. */
-  *keywords = NULL;
-
   /* Choose a property list to parse:  either the one that came into
      this function, or the one attached to PATH. */
   if (force_list == NULL)
@@ -205,21 +194,25 @@ svn_wc__get_keywords (svn_subst_keywords_t **keywords,
 
   /* The easy answer. */
   if (list == NULL)
-    return SVN_NO_ERROR;
+    {
+      *keywords = NULL;
+      return SVN_NO_ERROR;
+    }
 
   SVN_ERR (svn_wc_entry (&entry, path, adm_access, FALSE, pool));
 
-  SVN_ERR (svn_subst_build_keywords (&tmp_keywords,
-                                     list,
-                                     apr_psprintf (pool, "%ld",
-                                                   entry->cmt_rev),
-                                     entry->url,
-                                     entry->cmt_date,
-                                     entry->cmt_author,
-                                     pool));
+  SVN_ERR (svn_subst_build_keywords2 (keywords,
+                                      list,
+                                      apr_psprintf (pool, "%ld",
+                                                    entry->cmt_rev),
+                                      entry->url,
+                                      entry->cmt_date,
+                                      entry->cmt_author,
+                                      pool));
+
+  if (apr_hash_count (*keywords) == 0)
+    *keywords = NULL;
 
-  *keywords = apr_pmemdup (pool, &tmp_keywords, sizeof (tmp_keywords));
-      
   return SVN_NO_ERROR;
 }
 
@@ -263,3 +256,34 @@ svn_wc__maybe_set_executable (svn_boolean_t *did_set,
 
   return SVN_NO_ERROR;
 }
+
+
+svn_error_t *
+svn_wc__maybe_set_read_only (svn_boolean_t *did_set,
+                             const char *path,
+                             svn_wc_adm_access_t *adm_access,
+                             apr_pool_t *pool)
+{
+  const svn_string_t *needs_lock;
+  const svn_wc_entry_t* entry;
+
+  if (did_set)
+    *did_set = FALSE;
+
+  SVN_ERR (svn_wc_entry (&entry, path, adm_access, FALSE, pool));
+  if (entry && entry->lock_token)
+    return SVN_NO_ERROR;
+
+  SVN_ERR (svn_wc_prop_get (&needs_lock, SVN_PROP_NEEDS_LOCK, path, 
+                            adm_access, pool));
+
+  if (needs_lock != NULL)
+    {
+      SVN_ERR (svn_io_set_file_read_write_carefully (path, FALSE, 
+                                                     FALSE, pool));
+      if (did_set)
+        *did_set = TRUE;
+    }
+
+  return SVN_NO_ERROR;
+}
